@@ -7,25 +7,9 @@ from collections import defaultdict, Counter
 from functools import partial
 from itertools import chain
 
-from six import text_type
-
 from sacremoses.corpus import Perluniprops
 from sacremoses.util import parallelize_preprocess, grouper
 
-# Hack to enable Python2.7 to use encoding.
-import sys
-
-if sys.version_info[0] < 3:
-    import io
-    import warnings
-
-    open = io.open
-    warnings.warn(
-        str(
-            "You should really be using Python3!!! "
-            "Tick tock, tick tock, https://pythonclock.org/"
-        )
-    )
 
 perluniprops = Perluniprops()
 
@@ -38,9 +22,9 @@ class MosesTruecaser(object):
     """
 
     # Perl Unicode Properties character sets.
-    Lowercase_Letter = text_type("".join(perluniprops.chars("Lowercase_Letter")))
-    Uppercase_Letter = text_type("".join(perluniprops.chars("Uppercase_Letter")))
-    Titlecase_Letter = text_type("".join(perluniprops.chars("Uppercase_Letter")))
+    Lowercase_Letter = str("".join(perluniprops.chars("Lowercase_Letter")))
+    Uppercase_Letter = str("".join(perluniprops.chars("Uppercase_Letter")))
+    Titlecase_Letter = str("".join(perluniprops.chars("Uppercase_Letter")))
 
     def __init__(self, load_from=None, is_asr=None, encoding="utf8"):
         """
@@ -56,7 +40,7 @@ class MosesTruecaser(object):
         super(MosesTruecaser, self).__init__()
         # Initialize the language specific nonbreaking prefixes.
         self.SKIP_LETTERS_REGEX = re.compile(
-            u"[{}{}{}]".format(
+            "[{}{}{}]".format(
                 self.Lowercase_Letter, self.Uppercase_Letter, self.Titlecase_Letter
             )
         )
@@ -289,16 +273,12 @@ class MosesTruecaser(object):
             # The actual case replacement happens here.
             # "Most frequent" case of the word.
             best_case = self.model["best"].get(token.lower(), None)
-            # Other known cases of the word.
-            known_case = self.model["known"].get(token, None)
             # If it's the start of sentence.
             if is_first_word and best_case:  # Truecase sentence start.
                 token = best_case
-            elif known_case:  # Don't change known tokens.
-                token = known_case if use_known else token
-            elif (
-                best_case
-            ):  # Truecase otherwise unknown tokens? Heh? From https://github.com/moses-smt/mosesdecoder/blob/master/scripts/recaser/truecase.perl#L66
+            elif use_known and token in self.model["known"]:  # Don't change known tokens.
+                pass
+            elif best_case:  # Truecase otherwise unknown tokens? Heh? From https://github.com/moses-smt/mosesdecoder/blob/master/scripts/recaser/truecase.perl#L66
                 token = best_case
             # Else, it's an unknown word, don't change the word.
             # Concat the truecased `word` with the `other_factors`
@@ -307,9 +287,9 @@ class MosesTruecaser(object):
             truecased_tokens.append(token)
 
             # Resets sentence start if this token is an ending punctuation.
-            is_first_word = token in self.SENT_END
-
-            if token in self.DELAYED_SENT_START:
+            if token in self.SENT_END:
+                is_first_word = True
+            elif token not in self.DELAYED_SENT_START:
                 is_first_word = False
 
         # return ' '.join(tokens)
